import { DragNodeObject, DragAnyObject, DragNodeDataObject, DragObject, IPublicModelNode } from '@alilc/lowcode-types';
import { DropLocation } from './location';
import { Node, DocumentModel } from '../document';
import { NodeInstance, ComponentInstance } from '../simulator';
import { Designer } from './designer';
export interface LocateEvent {
    readonly type: 'LocateEvent';
    /**
     * 浏览器窗口坐标系
     */
    readonly globalX: number;
    readonly globalY: number;
    /**
     * 原始事件
     */
    readonly originalEvent: MouseEvent | DragEvent;
    /**
     * 拖拽对象
     */
    readonly dragObject: DragObject;
    /**
     * 激活的感应器
     */
    sensor?: ISensor;
    /**
     * 浏览器事件响应目标
     */
    target?: Element | null;
    /**
     * 当前激活文档画布坐标系
     */
    canvasX?: number;
    canvasY?: number;
    /**
     * 激活或目标文档
     */
    documentModel?: DocumentModel;
    /**
     * 事件订正标识，初始构造时，从发起端构造，缺少 canvasX,canvasY, 需要经过订正才有
     */
    fixed?: true;
}
/**
 * 拖拽敏感板
 */
export interface ISensor {
    /**
     * 是否可响应，比如面板被隐藏，可设置该值 false
     */
    readonly sensorAvailable: boolean;
    /**
     * 给事件打补丁
     */
    fixEvent(e: LocateEvent): LocateEvent;
    /**
     * 定位并激活
     */
    locate(e: LocateEvent): DropLocation | undefined | null;
    /**
     * 是否进入敏感板区域
     */
    isEnter(e: LocateEvent): boolean;
    /**
     * 取消激活
     */
    deactiveSensor(): void;
    /**
     * 获取节点实例
     */
    getNodeInstanceFromElement(e: Element | null): NodeInstance<ComponentInstance> | null;
}
export declare function isDragNodeObject(obj: any): obj is DragNodeObject;
export declare function isDragNodeDataObject(obj: any): obj is DragNodeDataObject;
export declare function isDragAnyObject(obj: any): obj is DragAnyObject;
export declare function isLocateEvent(e: any): e is LocateEvent;
/**
 * mouse shake check
 */
export declare function isShaken(e1: MouseEvent | DragEvent, e2: MouseEvent | DragEvent): boolean;
export declare function isInvalidPoint(e: any, last: any): boolean;
export declare function isSameAs(e1: MouseEvent | DragEvent, e2: MouseEvent | DragEvent): boolean;
export declare function setShaken(e: any): void;
/**
 * Drag-on 拖拽引擎
 */
export declare class Dragon {
    readonly designer: Designer;
    private sensors;
    /**
     * current active sensor, 可用于感应区高亮
     */
    private _activeSensor;
    get activeSensor(): ISensor | undefined;
    private _dragging;
    private _canDrop;
    get dragging(): boolean;
    private emitter;
    constructor(designer: Designer);
    /**
     * Quick listen a shell(container element) drag behavior
     * @param shell container element
     * @param boost boost got a drag object
     */
    from(shell: Element, boost: (e: MouseEvent) => DragObject | null): () => void;
    /**
     * boost your dragObject for dragging(flying) 发射拖拽对象
     *
     * @param dragObject 拖拽对象
     * @param boostEvent 拖拽初始时事件
     */
    boost(dragObject: DragObject, boostEvent: MouseEvent | DragEvent, fromRglNode?: Node | IPublicModelNode): void;
    private getMasterSensors;
    private getSimulators;
    private setNativeSelection;
    /**
     * 设置拖拽态
     */
    private setDraggingState;
    /**
     * 设置拷贝态
     */
    private setCopyState;
    /**
     * 清除所有态：拖拽态、拷贝态
     */
    private clearState;
    /**
     * 添加投放感应区
     */
    addSensor(sensor: any): void;
    /**
     * 移除投放感应
     */
    removeSensor(sensor: any): void;
    onDragstart(func: (e: LocateEvent) => any): () => void;
    onDrag(func: (e: LocateEvent) => any): () => void;
    onDragend(func: (x: {
        dragObject: DragObject;
        copy: boolean;
    }) => any): () => void;
}
