import { ReactElement } from 'react';
import { NodeSchema, PropsMap, PropsList, NodeData, I18nData, SlotSchema, PageSchema, ComponentSchema, NodeStatus, CompositeValue, GlobalEvent, IPublicModelNode } from '@alilc/lowcode-types';
import { SettingTopEntry } from '@alilc/lowcode-designer';
import { Props } from './props/props';
import { DocumentModel } from '../document-model';
import { NodeChildren } from './node-children';
import { Prop } from './props/prop';
import { ComponentMeta } from '../../component-meta';
import { ExclusiveGroup } from './exclusive-group';
import { TransformStage } from './transform-stage';
import { NodeRemoveOptions } from '../../types';
/**
 * 基础节点
 *
 * [Node Properties]
 *  componentName: Page/Block/Component
 *  props
 *  children
 *
 * [Directives]
 *  loop
 *  loopArgs
 *  condition
 *  ------- addition support -----
 *  conditionGroup use for condition, for exclusive
 *  title          display on outline
 *  ignored        ignore this node will not publish to render, but will store
 *  isLocked       can not select/hover/ item on canvas and outline
 *  hidden         not visible on canvas
 *  slotArgs       like loopArgs, for slot node
 *
 * 根容器节点
 *
 * [Node Properties]
 *  componentName: Page/Block/Component
 *  props
 *  children
 *
 * [Root Container Extra Properties]
 *  fileName
 *  meta
 *  state
 *  defaultProps
 *  dataSource
 *  lifeCycles
 *  methods
 *  css
 *
 * [Directives **not used**]
 *  loop
 *  loopArgs
 *  condition
 *  ------- future support -----
 *  conditionGroup
 *  title
 *  ignored
 *  isLocked
 *  hidden
 */
export declare class Node<Schema extends NodeSchema = NodeSchema> {
    readonly document: DocumentModel;
    private emitter;
    /**
     * 是节点实例
     */
    readonly isNode = true;
    /**
     * 节点 id
     */
    readonly id: string;
    /**
     * 节点组件类型
     * 特殊节点：
     *  * Page 页面
     *  * Block 区块
     *  * Component 组件/元件
     *  * Fragment 碎片节点，无 props，有指令
     *  * Leaf 文字节点 | 表达式节点，无 props，无指令？
     *  * Slot 插槽节点，无 props，正常 children，有 slotArgs，有指令
     */
    readonly componentName: string;
    /**
     * 属性抽象
     */
    props: Props;
    protected _children?: NodeChildren;
    /**
     * @deprecated
     */
    private _addons;
    private _parent;
    /**
     * 父级节点
     */
    get parent(): ParentalNode | null;
    /**
     * 当前节点子集
     */
    get children(): NodeChildren | null;
    /**
     * 当前节点深度
     */
    get zLevel(): number;
    get title(): string | I18nData | ReactElement;
    get icon(): any;
    isInited: boolean;
    constructor(document: DocumentModel, nodeSchema: Schema, options?: any);
    _settingEntry: SettingTopEntry;
    get settingEntry(): SettingTopEntry;
    /**
     * 节点初始化期间就把内置的一些 prop 初始化好，避免后续不断构造实例导致 reaction 执行多次
     */
    private initBuiltinProps;
    private initProps;
    private upgradeProps;
    private autoruns?;
    private setupAutoruns;
    private initialChildren;
    private _isRGLContainer;
    set isRGLContainer(status: boolean);
    get isRGLContainer(): boolean;
    isContainer(): boolean;
    isModal(): boolean;
    isRoot(): boolean;
    isPage(): boolean;
    isComponent(): boolean;
    isSlot(): boolean;
    /**
     * 是否一个父亲类节点
     */
    isParental(): this is ParentalNode;
    /**
     * 终端节点，内容一般为 文字 或者 表达式
     */
    isLeaf(): this is LeafNode;
    internalSetWillPurge(): void;
    private didDropIn;
    private didDropOut;
    /**
     * 内部方法，请勿使用
     * @param useMutator 是否触发联动逻辑
     */
    internalSetParent(parent: ParentalNode | null, useMutator?: boolean): void;
    private _slotFor?;
    internalSetSlotFor(slotFor: Prop | null | undefined): void;
    internalToShellNode(): IPublicModelNode | null;
    /**
     * 关联属性
     */
    get slotFor(): Prop;
    /**
     * 移除当前节点
     */
    remove(useMutator?: boolean, purge?: boolean, options?: NodeRemoveOptions): void;
    /**
     * 锁住当前节点
     */
    lock(flag?: boolean): void;
    /**
     * 获取当前节点的锁定状态
     */
    get isLocked(): boolean;
    /**
     * 选择当前节点
     */
    select(): void;
    /**
     * 悬停高亮
     */
    hover(flag?: boolean): void;
    /**
     * 节点组件描述
     */
    get componentMeta(): ComponentMeta;
    get propsData(): PropsMap | PropsList | null;
    _slots: Node[];
    hasSlots(): boolean;
    get slots(): Node<NodeSchema>[];
    private _conditionGroup;
    get conditionGroup(): ExclusiveGroup | null;
    setConditionGroup(grp: ExclusiveGroup | string | null): void;
    isConditionalVisible(): boolean | undefined;
    setConditionalVisible(): void;
    hasCondition(): boolean;
    /**
     * has loop when 1. loop is validArray with length > 1 ; OR  2. loop is variable object
     * @return boolean, has loop config or not
     */
    hasLoop(): boolean;
    wrapWith(schema: Schema): any;
    replaceWith(schema: Schema, migrate?: boolean): any;
    /**
     * 替换子节点
     *
     * @param {Node} node
     * @param {object} data
     */
    replaceChild(node: Node, data: any): Node;
    setVisible(flag: boolean): void;
    getVisible(): boolean;
    onVisibleChange(func: (flag: boolean) => any): () => void;
    getProp(path: string, createIfNone?: boolean): Prop | null;
    getExtraProp(key: string, createIfNone?: boolean): Prop | null;
    setExtraProp(key: string, value: CompositeValue): void;
    /**
     * 获取单个属性值
     */
    getPropValue(path: string): any;
    /**
     * 设置单个属性值
     */
    setPropValue(path: string, value: any): void;
    /**
     * 清除已设置的值
     */
    clearPropValue(path: string): void;
    /**
     * 设置多个属性值，和原有值合并
     */
    mergeProps(props: PropsMap): void;
    /**
     * 设置多个属性值，替换原有值
     */
    setProps(props?: PropsMap | PropsList | Props | null): void;
    /**
     * 获取节点在父容器中的索引
     */
    get index(): number;
    /**
     * 获取下一个兄弟节点
     */
    get nextSibling(): Node | null;
    /**
     * 获取上一个兄弟节点
     */
    get prevSibling(): Node | null;
    /**
     * 获取符合搭建协议-节点 schema 结构
     */
    get schema(): Schema;
    set schema(data: Schema);
    import(data: Schema, checkId?: boolean): void;
    toData(): Schema;
    /**
     * 导出 schema
     */
    export(stage?: TransformStage, options?: any): Schema;
    /**
     * 判断是否包含特定节点
     */
    contains(node: Node): boolean;
    /**
     * 获取特定深度的父亲节点
     */
    getZLevelTop(zLevel: number): Node | null;
    /**
     * 判断与其它节点的位置关系
     *
     *  16 thisNode contains otherNode
     *  8  thisNode contained_by otherNode
     *  2  thisNode before or after otherNode
     *  0  thisNode same as otherNode
     */
    comparePosition(otherNode: Node): PositionNO;
    unlinkSlot(slotNode: Node): boolean;
    /**
     * 删除一个Slot节点
     */
    removeSlot(slotNode: Node, purge?: boolean): boolean;
    addSlot(slotNode: Node): void;
    /**
     * 当前node对应组件是否已注册可用
     */
    isValidComponent(): boolean;
    /**
     * 删除一个节点
     * @param node
     */
    removeChild(node: Node): void;
    private purged;
    /**
     * 是否已销毁
     */
    get isPurged(): boolean;
    /**
     * 销毁
     */
    purge(): void;
    private purging;
    internalPurgeStart(): void;
    /**
     * 是否正在销毁
     */
    get isPurging(): boolean;
    /**
     * 是否可执行某action
     */
    canPerformAction(actionName: string): boolean;
    isEmpty(): boolean;
    getChildren(): NodeChildren;
    getComponentName(): string;
    insert(node: Node, ref?: Node, useMutator?: boolean): void;
    insertBefore(node: Node, ref?: Node, useMutator?: boolean): void;
    insertAfter(node: any, ref?: Node, useMutator?: boolean): void;
    getParent(): ParentalNode<NodeSchema>;
    getId(): string;
    getIndex(): number;
    getNode(): this;
    getRoot(): RootNode;
    getProps(): Props;
    onChildrenChange(fn: (param?: {
        type: string;
        node: Node;
    }) => void): (() => void) | undefined;
    mergeChildren(remover: () => any, adder: (children: Node[]) => NodeData[] | null, sorter: () => any): void;
    status: NodeStatus;
    /**
     * @deprecated
     */
    getStatus(field?: keyof NodeStatus): boolean | NodeStatus;
    /**
     * @deprecated
     */
    setStatus(field: keyof NodeStatus, flag: boolean): void;
    /**
     * @deprecated
     */
    getDOMNode(): any;
    /**
     * @deprecated
     */
    getPage(): DocumentModel;
    /**
     * 获取磁贴相关信息
     */
    getRGL(): any;
    /**
     * @deprecated
     */
    getSuitablePlace(node: Node, ref: any): any;
    /**
     * @deprecated
     */
    getAddonData(key: string): any;
    /**
     * @deprecated
     */
    registerAddon(key: string, exportData: () => any, isProp?: boolean): void;
    getRect(): DOMRect | null;
    /**
     * @deprecated
     */
    getPrototype(): any;
    /**
     * @deprecated
     */
    setPrototype(proto: any): void;
    getIcon(): any;
    toString(): string;
    emitPropChange(val: PropChangeOptions): void;
    onPropChange(func: (info: PropChangeOptions) => void): Function;
}
export interface ParentalNode<T extends NodeSchema = NodeSchema> extends Node<T> {
    readonly children: NodeChildren;
}
export interface LeafNode extends Node {
    readonly children: null;
}
export declare type PropChangeOptions = Omit<GlobalEvent.Node.Prop.ChangeOptions, 'node'>;
export declare type SlotNode = ParentalNode<SlotSchema>;
export declare type PageNode = ParentalNode<PageSchema>;
export declare type ComponentNode = ParentalNode<ComponentSchema>;
export declare type RootNode = PageNode | ComponentNode;
export declare function isNode(node: any): node is Node;
export declare function isRootNode(node: Node): node is RootNode;
export declare function isLowCodeComponent(node: Node): boolean;
export declare function getZLevelTop(child: Node, zLevel: number): Node | null;
/**
 * 测试两个节点是否为包含关系
 * @param node1 测试的父节点
 * @param node2 测试的被包含节点
 * @returns 是否包含
 */
export declare function contains(node1: Node, node2: Node): boolean;
export declare enum PositionNO {
    Contains = 16,
    ContainedBy = 8,
    BeforeOrAfter = 2,
    TheSame = 0
}
export declare function comparePosition(node1: Node, node2: Node): PositionNO;
export declare function insertChild(container: ParentalNode, thing: Node | NodeData, at?: number | null, copy?: boolean): Node;
export declare function insertChildren(container: ParentalNode, nodes: Node[] | NodeData[], at?: number | null, copy?: boolean): Node[];
