/// <reference types="node" />
/// <reference types="react" />
import { IReactionPublic, IReactionOptions, IReactionDisposer } from '@alilc/lowcode-editor-core';
import { EventEmitter } from 'events';
import { ISimulatorHost, Component, NodeInstance, ComponentInstance, DropContainer } from '../simulator';
import Viewport from './viewport';
import { Node } from '../document';
import ResourceConsumer from './resource-consumer';
import { Asset, AssetList, UtilsMetadata } from '@alilc/lowcode-utils';
import { LocateEvent, Rect, Designer } from '../designer';
import { ComponentMetadata, ComponentSchema, Package } from '@alilc/lowcode-types';
import { BuiltinSimulatorRenderer } from './renderer';
import { LiveEditing } from './live-editing/live-editing';
import { Project } from '../project';
import { Scroller } from '../designer/scroller';
export interface LibraryItem extends Package {
    package: string;
    library: string;
    urls?: Asset;
    editUrls?: Asset;
}
export interface DeviceStyleProps {
    canvas?: object;
    viewport?: object;
}
export interface BuiltinSimulatorProps {
    designMode?: 'live' | 'design' | 'preview' | 'extend' | 'border';
    device?: 'mobile' | 'iphone' | string;
    deviceClassName?: string;
    environment?: Asset;
    /** @property 请求处理器配置 */
    requestHandlersMap?: any;
    extraEnvironment?: Asset;
    library?: LibraryItem[];
    utilsMetadata?: UtilsMetadata;
    simulatorUrl?: Asset;
    theme?: Asset;
    componentsAsset?: Asset;
    [key: string]: any;
}
export declare class BuiltinSimulatorHost implements ISimulatorHost<BuiltinSimulatorProps> {
    readonly isSimulator = true;
    readonly project: Project;
    readonly designer: Designer;
    readonly viewport: Viewport;
    readonly scroller: Scroller;
    readonly emitter: EventEmitter;
    readonly componentsConsumer: ResourceConsumer;
    readonly injectionConsumer: ResourceConsumer;
    readonly i18nConsumer: ResourceConsumer;
    /**
     * 是否为画布自动渲染
     */
    autoRender: boolean;
    stopAutoRepaintNode(): void;
    enableAutoRepaintNode(): void;
    constructor(project: Project);
    get currentDocument(): import("../document").DocumentModel;
    get renderEnv(): string;
    get device(): string;
    get locale(): string;
    get deviceClassName(): string | undefined;
    get designMode(): 'live' | 'design' | 'preview';
    get requestHandlersMap(): any;
    get thisRequiredInJSE(): any;
    get enableStrictNotFoundMode(): any;
    get notFoundComponent(): any;
    get faultComponent(): any;
    get componentsAsset(): Asset | undefined;
    get theme(): Asset | undefined;
    get componentsMap(): {
        [key: string]: object | import("react").ComponentClass<any, any> | import("react").FunctionComponent<any> | import("@alilc/lowcode-types").NpmInfo;
    };
    get deviceStyle(): DeviceStyleProps | undefined;
    _props: BuiltinSimulatorProps;
    /**
     * @see ISimulator
     */
    setProps(props: BuiltinSimulatorProps): void;
    set(key: string, value: any): void;
    get(key: string): any;
    /**
     * 有 Renderer 进程连接进来，设置同步机制
     */
    connect(renderer: BuiltinSimulatorRenderer, effect: (reaction: IReactionPublic) => void, options?: IReactionOptions): IReactionDisposer;
    reaction(expression: (reaction: IReactionPublic) => unknown, effect: (value: unknown, prev: unknown, reaction: IReactionPublic) => void, opts?: IReactionOptions | undefined): IReactionDisposer;
    autorun(effect: (reaction: IReactionPublic) => void, options?: IReactionOptions): IReactionDisposer;
    purge(): void;
    mountViewport(viewport: Element | null): void;
    private _contentWindow?;
    get contentWindow(): Window;
    private _contentDocument?;
    get contentDocument(): Document;
    private _renderer?;
    get renderer(): BuiltinSimulatorRenderer;
    readonly asyncLibraryMap: {
        [key: string]: {};
    };
    readonly libraryMap: {
        [key: string]: string;
    };
    private _iframe?;
    /**
     * {
     *   "title":"BizCharts",
     *   "package":"bizcharts",
     *   "exportName":"bizcharts",
     *   "version":"4.0.14",
     *   "urls":[
     *      "https://g.alicdn.com/code/lib/bizcharts/4.0.14/BizCharts.js"
     *   ],
     *   "library":"BizCharts"
     * }
     * package：String 资源npm包名
     * exportName：String umd包导出名字，用于适配部分物料包define name不一致的问题，例如把BizCharts改成bizcharts，用来兼容物料用define声明的bizcharts
     * version：String 版本号
     * urls：Array 资源cdn地址，必须是umd类型，可以是.js或者.css
     * library：String umd包直接导出的name
     */
    buildLibrary(library?: LibraryItem[]): AssetList;
    rerender(): void;
    mountContentFrame(iframe: HTMLIFrameElement | null): Promise<void>;
    setupComponents(library: any): Promise<void>;
    setupEvents(): void;
    postEvent(eventName: string, ...data: any[]): void;
    setupDragAndClick(): void;
    private disableHovering?;
    private disableDetecting?;
    /**
     * 设置悬停处理
     */
    setupDetecting(): void;
    readonly liveEditing: LiveEditing;
    setupLiveEditing(): void;
    /**
     * @see ISimulator
     */
    setSuspense(suspended: boolean): boolean;
    setupContextMenu(): void;
    /**
     * @see ISimulator
     */
    generateComponentMetadata(componentName: string): ComponentMetadata;
    /**
     * @see ISimulator
     */
    getComponent(componentName: string): Component | null;
    createComponent(schema: ComponentSchema): Component | null;
    private instancesMap;
    setInstance(docId: string, id: string, instances: ComponentInstance[] | null): void;
    /**
     * @see ISimulator
     */
    getComponentInstances(node: Node, context?: NodeInstance): ComponentInstance[] | null;
    /**
     * @see ISimulator
     */
    getComponentContext(): any;
    /**
     * @see ISimulator
     */
    getClosestNodeInstance(from: ComponentInstance, specId?: string): NodeInstance<ComponentInstance> | null;
    /**
     * @see ISimulator
     */
    computeRect(node: Node): Rect | null;
    /**
     * @see ISimulator
     */
    computeComponentInstanceRect(instance: ComponentInstance, selector?: string): Rect | null;
    /**
     * @see ISimulator
     */
    findDOMNodes(instance: ComponentInstance, selector?: string): Array<Element | Text> | null;
    /**
     * 通过 DOM 节点获取节点，依赖 simulator 的接口
     */
    getNodeInstanceFromElement(target: Element | null): NodeInstance<ComponentInstance> | null;
    private tryScrollAgain;
    /**
     * @see ISimulator
     */
    scrollToNode(node: Node, detail?: any): void;
    /**
     * @see ISimulator
     */
    setNativeSelection(enableFlag: boolean): void;
    /**
     * @see ISimulator
     */
    setDraggingState(state: boolean): void;
    /**
     * @see ISimulator
     */
    setCopyState(state: boolean): void;
    /**
     * @see ISimulator
     */
    clearState(): void;
    private _sensorAvailable;
    /**
     * @see ISensor
     */
    get sensorAvailable(): boolean;
    /**
     * @see ISensor
     */
    fixEvent(e: LocateEvent): LocateEvent;
    /**
     * @see ISensor
     */
    isEnter(e: LocateEvent): boolean;
    private sensing;
    /**
     * @see ISensor
     */
    deactiveSensor(): void;
    /**
     * @see ISensor
     */
    locate(e: LocateEvent): any;
    /**
     * 查找合适的投放容器
     */
    getDropContainer(e: LocateEvent): DropContainer | null;
    isAcceptable(): boolean;
    /**
     * 控制接受
     */
    handleAccept({ container, instance }: DropContainer, e: LocateEvent): boolean;
    /**
     * 查找邻近容器
     */
    getNearByContainer({ container, instance }: DropContainer, drillDownExcludes: Set<Node>, e: LocateEvent): any;
}
